/*
 * Copyright (c) 1996 by Argonaut Technologies Limited. All rights reserved.
 *
 * $Id: PSYQFILE.C 1.1 1996/09/12 14:01:14 NIKH Exp $
 * $Locker: $
 *
 * File handler that uses PSYQ fileserver functions
 */

#include <sys/types.h>
#include <r3000.h>
#include <asm.h>
#include <kernel.h>
#include <libgte.h>
#include <libgpu.h>
#include <libetc.h>
#include <libsn.h>

#include <stdio.h>

#include "brender.h"

static char rscid[] = "$Id: PSYQFILE.C 1.1 1996/09/12 14:01:14 NIKH Exp $";

#define FILENAME_MAX 80
#define FILEREAD	0
#define FILEWRITE	1
#define FILEREADWRITE	2
#define FILEERROR 	-1

#define FILESTART	0
#define FILECURRENT	1
#define FILEEND		2

#define PRINT_LOG	0

static long file_length,file_ptr;
static char *file_buffer;

static br_uint_32 BR_CALLBACK PsyqAttributes(void)
{
	return 
		BR_FS_ATTR_READABLE   | 
		BR_FS_ATTR_WRITEABLE  |
		BR_FS_ATTR_HAS_TEXT   |
		BR_FS_ATTR_HAS_BINARY |
		BR_FS_ATTR_HAS_ADVANCE;
}

/*
 * Open a file for reading
 *
 * Having found the file, invoke a supplied callback, (if present)
 * to find out if the file is text, binary, or unknown
 *
 * Text mode files are primarily used for debugging but it can be
 * useful to allow hand editing of input data
 *
 * Return a void * file handle ('FILE *' cast to 'void *') or NULL
 * if open failed
 */
void * BR_CALLBACK PsyqOpenRead(const char *name, br_size_t n_magics,
		br_mode_test_cbfn *identify,
		int *mode_result)
{
	void *fh;
	char *br_path;
   	char try_name[FILENAME_MAX+1],*cp;
	br_uint_8 magics[BR_MAX_FILE_MAGICS];
	int open_mode = BR_FS_MODE_BINARY;

	/*
	 * Try the current directory
	 */
	memcpy(try_name,name,FILENAME_MAX);

	if((fh = (void *)malloc(sizeof(int))) == NULL)
		BR_ERROR1("Unable to allocate %d bytes",sizeof(int));

	if((*((int *)fh) = PCopen(try_name,FILEREAD,0)) == FILEERROR)
		return NULL;
	else {
		file_length = PClseek(*((int *)fh),0,FILEEND);
		file_ptr = 0;
		PClseek(*((int *)fh),0,FILESTART);
	}

	/*
	 * Now have an open file, try and grab the first bytes from it
	 */
	if((PCread(*((int *)fh), magics, n_magics)) != n_magics) {
	    	/*
		 * could not read all of required data, close, die
		 */
		PCclose(*((int *)fh));
		free(fh);
		return NULL;
	}

	/*
	 * Try and identify the file
	 */
	if(identify)
		open_mode = identify(magics, n_magics);

	if(mode_result)
		*mode_result = open_mode;

	/*
	 * Reopen file with it's new identity (or abandon if unknown identity)
	 */
	switch(open_mode) {
	case BR_FS_MODE_TEXT:
		PCclose(*((int *)fh));

		*((int *)fh) = PCopen(try_name,FILEREAD,0);
		file_length = PClseek(*((int *)fh),0,FILEEND);
		file_ptr = 0;
		PClseek(*((int *)fh),0,FILESTART);
		
		file_buffer = (char *)malloc(file_length);
		PCread(*((int *)fh),file_buffer,file_length);
		PCclose(*((int *)fh));
		*((int *)fh) = 0;
		
		break;

	case BR_FS_MODE_BINARY:
		PCclose(*((int *)fh));

		*((int *)fh) = PCopen(try_name,FILEREAD,0);
		file_length = PClseek(*((int *)fh),0,FILEEND);
		file_ptr = 0;
		PClseek(*((int *)fh),0,FILESTART);

		file_buffer = (char *)malloc(file_length);
		PCread(*((int *)fh),file_buffer,file_length);
		PCclose(*((int *)fh));

		*((int *)fh) = 0;
		break;

	case BR_FS_MODE_UNKNOWN:
		PCclose(*((int *)fh));
		free(fh);
		fh = NULL;
		break;

	default:
		BR_ERROR1("BrStdFileOpenRead: invalid open_mode %d",open_mode);
	}

	return fh;
}



/*
 * Open a file for writing, overwrites any existing file of the same name
 *
 * Return a void * file handle ('INT *' cast to 'void *') or NULL
 * if open failed
 */
static void * BR_CALLBACK PsyqOpenWrite(const char *name, int mode)
{
	void *fh;
	
	/*
	 * PS allocate memory for file handle (integer)
	 */
	if((fh = (char *)malloc(sizeof(int))) == NULL)
		BR_ERROR1("Unable to allocate %d bytes",sizeof(int));

	if((*((int *)fh) = PCcreat(name,0)) == FILEERROR)
		fh = NULL;
	else {
	    	file_length = 0;
		file_ptr = 0;
	}
	
	return fh;
}

/*
 * Close an open file
 */
static void BR_CALLBACK PsyqClose(void *f)
{
	file_ptr = 1;
	file_length = 0;

	if(file_buffer != NULL) {
		free(file_buffer);
		file_buffer = NULL;
	}

	if(f != NULL)
		if(*((int *)f) != 0)
			PCclose(*((int *)f));

	free(f);
}

/*
 * Test EOF
 */
static int BR_CALLBACK PsyqEof(const void *f)
{
	return (file_ptr >= file_length);
}

/*
 * Read one character from file
 */
static int BR_CALLBACK PsyqGetChar(void *f)
{
	char ch;

	if(file_ptr >= file_length)
		return BR_EOF;
	
	memcpy(&ch,file_buffer + file_ptr++,sizeof(char));
	return (int)ch;
}


/*
 * Write one character to file
 */
static void BR_CALLBACK PsyqPutChar(int c, void *f)
{
    	char ch = c;
#if PRINT_LOG
	printf("%c",ch);
#else	
	PCwrite(*((int *)f),&ch,1);
#endif	
	++file_ptr;
}

/*
 * Read a block from a file
 */
static br_size_t BR_CALLBACK PsyqRead(void *buf, br_size_t size, unsigned int n, void *f)
{
	memcpy(buf,file_buffer + file_ptr,size*n);
	file_ptr += size*n;
	return size*n;
}

/*
 * Write a block to a file
 */
static br_size_t BR_CALLBACK PsyqWrite(void *buf, br_size_t size, unsigned int n, void *f)
{    
	br_uint_32 count;
	int i;

	file_ptr += size*n;
#if PRINT_LOG

	for(i=0; i<size*n; i++)
		printf("%c", ((char *)buf)[i]);
		
	count = size * n;
#else	
	count = PCwrite(*((int *)f),buf,size*n);
#endif
	return count;
}

/*
 * Read a line of text and trim any terminators
 *
 * Return length of line
 */
static br_size_t BR_CALLBACK PsyqGetLine(char *buf, br_size_t buf_len, void * f)
{
	br_size_t l;
	
	l = 0;
	buf[0] = 0;

	do {
		buf[l] = file_buffer[file_ptr++];
		if(buf[l] == '\n')
			buf[l] = '\0';
	}
	while ((buf[l] != '\0') && (l++ < buf_len) && (PsyqEof(f) != 1));

	return l;
}

/*
 * Write a line to text file, followed by newline
 */
void BR_CALLBACK PsyqPutLine(const char *buf, void * f)
{
	char ch = '\n';
	br_size_t l;

	l = 0;

	while(buf[l] != '\0') {
	
#if PRINT_LOG
		printf("%c", buf[l]);
#else
		PCwrite(*((int *)f),buf + l, 1);
#endif
		++file_ptr;
		++l;
	}
#if PRINT_LOG
	printf("\n");
#else	
	PCwrite(*((int *)f), &ch, 1);
#endif
}

/*
 * Advance N bytes through a binary stream
 */
static void BR_CALLBACK PsyqAdvance(br_size_t count, void *f)
{
	file_ptr += count;
}

/*
 * Filesystem structure
 */
br_filesystem PsyqFilesystem={

	"Psyq PS",			/* identifier	*/
	
	PsyqAttributes,			/* attributes	*/
	PsyqOpenRead,			/* open_read	*/
	PsyqOpenWrite,			/* openwrite	*/
	PsyqClose,			/* close	*/
	PsyqEof,			/* eof		*/

	PsyqGetChar,			/* getchar	*/
	PsyqPutChar,			/* putchar	*/

	PsyqRead,			/* read		*/
	PsyqWrite,			/* write	*/

	PsyqGetLine,			/* getline	*/
	PsyqPutLine,			/* putline	*/

	PsyqAdvance,			/* advance	*/

};

